#include <string>
#include <list>
#include <map>
#include <fstream>

#define INFINITY 100000

// This version of bfs uses a pointer representation.
// Each vertex contains within it a list of pointers
// to the other vertices the vertex is connected to.
template <typename T> class vertex
{
public:
	T key;
	enum color {white,gray,black};
	color c;
	int d;
	vertex<T> *p;
	std::list<vertex<T>*> edges;

	vertex() {}
	vertex(const T& v) { key = v; c = white; d = INFINITY; p = NULL; }
};

template <typename T> class graph
{
public:
	graph(const std::string& fileName);
	~graph();

	void bfs(const T& s);
	
	void pathTo(const T& dest,std::list<T>& L);
private:
	// Since the edges are stored in the vertices, we only need
	// a list of vertices in the graph. For convenience, this is
	// stored as a map.
	std::map<T,vertex<T>*> V;
};

template <typename T>
graph<T>::graph(const std::string& filename)
{
	std::ifstream in;
	in.open(filename.c_str());
	
	int vertex_count = 0;
	in >> vertex_count;
	for(int n = 0;n < vertex_count;n++) {
		T key;
		in >> key;
		V[key] = new vertex<T>(key);
	}

	int edge_count = 0;
	in >> edge_count;
	for(int n = 0;n < edge_count;n++) {
		T first, second;
		in >> first >> second;
		V[first]->edges.push_back(V[second]);
	}
}

template <typename T>
graph<T>::~graph() {
	auto itr = V.begin();
	while(itr != V.end()) {
		delete itr->second;
		itr++;
	}
}

template <typename T>
void graph<T>::bfs(const T& s) {
	auto itr = V.begin();
	while(itr != V.end()) {
		if(itr->first != s) {
			itr->second->c = vertex<T>::white;
			itr->second->d = INFINITY;
			itr->second->p = NULL;
		} else {
			itr->second->c = vertex<T>::gray;
			itr->second->d = 0;
			itr->second->p = itr->second;
		}
		itr++;
	}

	// This version requires only a single vertex lookup. We look up the
	// start vertex in the map. From that point forward everything gets
	// done with pointers.
	std::list<vertex<T>*> Q;
	Q.push_back(V[s]);

	while(Q.size() != 0) {
		vertex<T> *u = Q.front();
		Q.pop_front();

		std::list<vertex<T>*>& edges = u->edges;
		auto itr2 = edges.begin();
		while(itr2 != edges.end()) {
			vertex<T>* v = *itr2;
			if(v->c == vertex<T>::white) {
				v->c = vertex<T>::gray;
				v->d = u->d + 1;
				v->p = u;
				Q.push_back(v);
			}
			itr2++;
		}
		u->c = vertex<T>::black;
	}
}

template <typename T>
void graph<T>::pathTo(const T& dest,std::list<T>& L)
{
	vertex<T> *v = V[dest];
	if(v->d == 0)
		L.push_back(dest);
	else if(v->p == NULL)
		return;
	else {
		this->pathTo(v->p->key,L);
		L.push_back(dest);
	}
}
